//Grenader.cs;

datablock AudioProfile(GrenaderFireSound)
{
   filename    = "./grenade_launcher.wav";
   description = AudioDefault3d;
   preload = true;
};

datablock ParticleData(GrenadeSmokeParticle)
{
	dragCoefficient      = 5;
	gravityCoefficient   = -0.5;
	inheritedVelFactor   = 0.2;
	constantAcceleration = 0.0;
	lifetimeMS           = 1200;
	lifetimeVarianceMS   = 350;
	textureName          = "base/data/particles/cloud";
	spinSpeed		= 10.0;
	spinRandomMin		= -500.0;
	spinRandomMax		= 500.0;

   colors[0]     = "0.5 0.5 0.5 0.0";
	colors[1]     = "0.5 0.5 0.5 0.1";
	colors[2]     = "0.5 0.5 0.5 0.0";

	sizes[0]      = 1.55;
   sizes[1]      = 0.3;
	sizes[2]      = 0.0;

   times[0] = 0.0;
   times[1] = 0.38;
   times[2] = 1.0;

	useInvAlpha = false;
};
datablock ParticleEmitterData(GrenadeSmokeEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 2.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 25;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "GrenadeSmokeParticle";
};

datablock ExplosionData(GrenaderExplosion : RPGExplosion)
{
   //explosionShape = "";

   explosionShape = "add-ons/weapon_rocket_launcher/explosionSphere1.dts";

   damageRadius = 15;
   radiusDamage = 60;

   impulseRadius = 16;
   impulseForce = 3000;
};

AddDamageType("GrenaderDirect",   '<bitmap:add-ons/Weapon_Package_Explosive2/CI_Grenade_launcher> %1',    '%2 <bitmap:add-ons/Weapon_Package_Explosive2/CI_Grenade_launcher> %1',1,1);
datablock ProjectileData(GrenaderProjectile)
{
   projectileShapeName = "./grenade_projectile.2.dts";
   directDamage        = 100;
   directDamageType = $DamageType::GrenaderDirect;
   radiusDamageType = $DamageType::GrenaderDirect;
   impactImpulse	   = 1000;
   verticalImpulse	   = 1000;
   explosion           = GrenaderExplosion;
   particleEmitter     = GrenadeSmokeEmitter;

   brickExplosionRadius = 3;
   brickExplosionImpact = false;          //destroy a brick if we hit it directly?
   brickExplosionForce  = 50;             
   brickExplosionMaxVolume = 76;          //max volume of bricks that we can destroy
   brickExplosionMaxVolumeFloating = 98;  //max volume of bricks that we can destroy if they aren't connected to the ground (should always be >= brickExplosionMaxVolume)

   muzzleVelocity      = 95;
   velInheritFactor    = 0.0;

   armingDelay         = 0;
   lifetime            = 3500;
   fadeDelay           = 3500;
   bounceElasticity    = 0.7;
   bounceFriction      = 0.40;
   isBallistic         = true;
   gravityMod = 1;
   explodeOnPlayerImpact = true;
   explodeondeath = true;

   hasLight    = false;
   lightRadius = 5.0;
   lightColor  = "1 0.5 0.0";

   uiName = "GL Grenade";
};

//////////
// item //
//////////
datablock ItemData(GrenaderItem)
{
	category = "Weapon";  // Mission editor category
	className = "Weapon"; // For inventory system

	 // Basic Item Properties
   shapeFile = "./M79.DTS";
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Grenade Launcher";
	iconName = "./grenadelauncher";
	doColorShift = false;
	colorShiftColor = "0.40 0.40 0.42 1.000";

	 // Dynamic properties defined by the scripts
	image = GrenaderImage;
	canDrop = true;

	TT_ammoType = "Bomb";
	TT_reloads = true;
	TT_maxAmmo = 1;
	TT_alwaysReloadPref = "Ex";
};

////////////////
//weapon image//
////////////////
datablock ShapeBaseImageData(GrenaderImage)
{
   // Basic Item properties
   shapeFile = "./M79.DTS";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
   eyeOffset = 0; //"0.7 1.2 -0.5";
   rotation = eulerToMatrix( "0 0 0" );

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = GrenaderItem;
   ammo = " ";
   projectile = GrenaderProjectile;
   projectileType = Projectile;

   //melee particles shoot from eye node for consistancy
   melee = false;
   //raise your arm up or not
   armReady = true;
   minShotTime = 0; // 700

   doColorShift = true;
   colorShiftColor = GrenaderItem.colorShiftColor;//"0.400 0.196 0 1.000";

   //casing = " ";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
	stateName[0]                     = "Activate";
	stateTimeoutValue[0]             = 0.35;
	stateSound[0]					= weaponSwitchSound;
	stateTransitionOnTimeout[0]       = "LoadCheckA";

	stateName[1]                     = "Ready";
	stateTransitionOnTriggerDown[1]  = "FireCheckA";
	stateTransitionOnNotLoaded[1]		= "ReloadStart";

	stateName[2]                    = "Fire";
	stateTransitionOnTimeout[2]     = "Delay";
	stateTimeoutValue[2]            = 0.01;
	stateFire[2]                    = true;
	stateAllowImageChange[2]        = false;
	stateEmitter[2]			  = shotgunExplosionRingEmitter;
	stateEmitterTime[2]		  = 0.03;
	stateScript[2]                  = "onFire";
	stateSound[2]					= GrenaderFireSound;

	stateName[3]			= "Delay";
	stateTransitionOnTimeout[3]     = "ReloadStart";
	stateTimeoutValue[3]            = 0.1;
	
	stateName[4]				= "LoadCheckA";
	stateScript[4]				= "TT_onLoadCheck";
	stateTimeoutValue[4]			= 0.01;
	stateTransitionOnTimeout[4]		= "LoadCheckB";
	
	stateName[5]				= "LoadCheckB";
	stateTransitionOnLoaded[5]		= "Ready";
	stateTransitionOnNotLoaded[5]		= "Empty";

	stateName[6]				= "Reload";
	stateTimeoutValue[6]			= 1.2;
	stateScript[6]				= "onReloadStart";
	stateTransitionOnTimeout[6]		= "Wait";
	
	stateName[7]				= "Wait";
	stateTimeoutValue[7]			= 0.4;
	stateScript[7]				= "onReloadWait";
	stateTransitionOnTimeout[7]		= "Reloaded";
	
	stateName[8]				= "FireLoadCheckA";
	stateScript[8]				= "TT_onLoadCheck";
	stateTimeoutValue[8]			= 0.01;
	stateTransitionOnTimeout[8]		= "FireLoadCheckB";
	
	stateName[9]				= "FireLoadCheckB";
	stateTransitionOnLoaded[9]		= "Ready";
	stateTransitionOnAmmo[9]		= "Reload";
	stateTransitionOnNoAmmo[9]    = "Empty";

	stateName[10]                   = "Empty";
	stateTransitionOnLoaded[10]     = "Ready";
	stateTransitionOnAmmo[10]  = "ReloadStart";
	stateTransitionOnTriggerDown[10]     = "FireCheckA";

	stateName[11]                    = "EmptyFire";
	stateScript[11]                  = "TT_onEmptyFire";
	stateTransitionOnLoaded[11]      = "Ready";
	stateTransitionOnAmmo[11]        = "ReloadStart";
	stateTransitionOnTriggerUp[11]   = "Empty";
	
	stateName[12]				= "Reloaded";
	stateTimeoutValue[12]			= 0.1;
	stateScript[12]				= "onReloaded";
	stateTransitionOnTimeout[12]		= "Activate";

	stateName[13]                    = "FireCheckA";
	stateScript[13]                  = "TT_onFireCheck";
	stateTransitionOnTimeout[13]     = "FireCheckB";

	stateName[14]                    = "FireCheckB";
	stateTransitionOnLoaded[14]   = "Fire";
	stateTransitionOnNotLoaded[14]      = "EmptyFire";

	stateName[15]                     = "ReloadStart";
	stateTransitionOnTimeout[15]  = "FireLoadCheckA";
	stateTimeoutValue[15]            = 0.1;
	stateAllowImageChange[15]         = false;
};

function GrenaderImage::onFire(%this,%obj,%slot)
{
	%this.TT_decrementAmmo(%obj);
	%this.TT_displayAmmo(%obj);
	%obj.playThread(2, plant);
	return Parent::onFire(%this,%obj,%slot);
}

function GrenaderImage::onReloadStart(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return;
	%obj.playThread(2, shiftDown);
	serverPlay3D(block_PlantBrick_Sound,%obj.getPosition());
	%this.TT_displayAmmo(%obj);
}

function GrenaderImage::onReloadWait(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return;
	%obj.playThread(2, plant);
	serverPlay3D(block_MoveBrick_Sound,%obj.getPosition());
	%this.TT_displayAmmo(%obj);
}

function GrenaderImage::onReloaded(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return %obj.setImageLoaded(%slot, 1);
	%this.TT_reload(%obj, %slot, block_PlantBrick_Sound);
	%this.TT_displayAmmo(%obj);
}
