//audio
datablock AudioProfile(SniperCarbineFireSound)
{
   filename    = "./SniperCarbine_fire.wav";
   description = AudioClose3d;
   preload = true;
};

datablock ParticleData(SniperCarbineTrailParticle)
{
	dragCoefficient		= 3.0;
	windCoefficient		= 0.0;
	gravityCoefficient	= 0.0;
	inheritedVelFactor	= 0.0;
	constantAcceleration	= 0.0;
	lifetimeMS		= 220;
	lifetimeVarianceMS	= 0;
	spinSpeed		= 10.0;
	spinRandomMin		= -50.0;
	spinRandomMax		= 50.0;
	useInvAlpha		= false;
	animateTexture		= false;
	//framesPerSec		= 1;

	textureName		= "base/data/particles/dot";
	//animTexName		= "~/data/particles/dot";

	// Interpolation variables
	colors[0]	= "1 1 1 1";
	colors[1]	= "1 1 0.4 1";
	colors[2]	= "1 1 0 1";
	sizes[0]	= 0.05;
	sizes[1]	= 0.15;
	sizes[2]	= 0.0;
	times[0]	= 0.0;
	times[1]	= 0.5;
	times[2]	= 1.0;
};

datablock ParticleEmitterData(SniperCarbineTrailEmitter)
{
   ejectionPeriodMS = 2;
   periodVarianceMS = 0;

   ejectionVelocity = 0; //0.25;
   velocityVariance = 0; //0.10;

   ejectionOffset = 0;

   thetaMin         = 0.0;
   thetaMax         = 90.0;  

   particles = SniperCarbineTrailParticle;

   useEmitterColors = true;
};

datablock ProjectileData(SniperCarbineTracerProjectile)
{
   directDamage        = 0;
   directDamageType    = $DamageType::Gun;
   radiusDamageType    = $DamageType::Gun;

   brickExplosionRadius = 0;
   brickExplosionImpact = true;          //destroy a brick if we hit it directly?
   brickExplosionForce  = 10;
   brickExplosionMaxVolume = 1;          //max volume of bricks that we can destroy
   brickExplosionMaxVolumeFloating = 2;  //max volume of bricks that we can destroy if they aren't connected to the ground

   impactImpulse	     = 100;
   verticalImpulse	  = 50;
   explosion           = gunExplosion;
   particleEmitter     = "SniperCarbineTrailEmitter";

   muzzleVelocity      = 200;
   velInheritFactor    = 0;

   armingDelay         = 0;
   lifetime            = 400;
   fadeDelay           = 400;
   bounceElasticity    = 0.5;
   bounceFriction      = 0.10;
   isBallistic         = true;
   gravityMod = 0.0;

   hasLight    = false;
   lightRadius = 3.0;
   lightColor  = "0 0 0.5";
};

//////////
// item //
//////////
datablock ItemData(SniperCarbineItem)
{
	category = "Weapon";  // Mission editor category
	className = "Weapon"; // For inventory system

	 // Basic Item Properties
	shapeFile = "./sniper_carbine.dts";
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Sniper Carbine";
	iconName = "./snipercarbine";
	doColorShift = true;
	colorShiftColor = "0.33 0.33 0.33 1.000";

	 // Dynamic properties defined by the scripts
	image = SniperCarbineImage;
	canDrop = true;

	TT_ammoType = "708";
	TT_reloads = true;
	TT_maxAmmo = 11;
};

AddDamageType("SniperCarbine",   '<bitmap:add-ons/Weapon_Package_Tier2A/snipercarb_ci> %1',    '%2 <bitmap:add-ons/Weapon_Package_Tier2A/snipercarb_ci> %1',0.75,1);

////////////////
//weapon image//
////////////////
datablock ShapeBaseImageData(SniperCarbineImage)
{

   // Basic Item properties
   shapeFile = "./sniper_carbine.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
   eyeOffset = 0; //"0.7 1.2 -0.5";
   rotation = eulerToMatrix( "0 0 0" );

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = SniperCarbineItem;
   ammo = " ";
   projectile = SniperCarbineTracerProjectile;
   projectileType = Projectile;

   casing = GunShellDebris;
   shellExitDir        = "1.0 0.1 1.0";
   shellExitOffset     = "0 0 0";
   shellExitVariance   = 10.0;	
   shellVelocity       = 5.0;

   //melee particles shoot from eye node for consistancy
   melee = false;
   //raise your arm up or not
   armReady = true;

   doColorShift = true;
   colorShiftColor = SniperCarbineItem.colorShiftColor;

   // hook into Support_ImageAltTrigger
   altTriggerEnabled = true;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.
   TT_raycastEnabled = true;
   TT_raycastWeaponRange = 70; //varies
   TT_raycastWeaponTargets =
                   $TypeMasks::PlayerObjectType |    //AI/Players
                   $TypeMasks::StaticObjectType |    //Static Shapes
                   $TypeMasks::TerrainObjectType |    //Terrain
                   $TypeMasks::VehicleObjectType |    //Terrain
                   $TypeMasks::FXBrickObjectType;    //Bricks
   TT_raycastExplosionProjectile = GunProjectile;
   TT_raycastExplosionBrickSound = bulletHitSound;
   TT_raycastExplosionPlayerSound = bulletHitSound;
   TT_raycastDirectDamage = 12; //10
   TT_raycastDirectDamageType = $DamageType::SniperCarbine;
   TT_raycastSpreadAmt = 0.0028; //varies
   TT_raycastSpreadCount = 1;
   TT_raycastTracerProjectile = SniperCarbineTracerProjectile;
   TT_raycastFromMuzzle = true;

   // Initial start up state
	stateName[0]                    = "Activate";
	stateTimeoutValue[0]            = 0.15;
	stateSequence[0]		= "Activate";
	stateTransitionOnTimeout[0]     = "LoadCheckA";
	stateSound[0]			= weaponSwitchSound;

	stateName[1]                    = "Ready";
	stateTransitionOnNotLoaded[1]	= "ReloadStart";
	stateTransitionOnTriggerDown[1] = "FireCheckA";
	// stateScript[1]                  = "onReady";
	// stateSequence[1]			  = "ready";

	stateName[2]                    = "Fire";
	stateTransitionOnTimeout[2]     = "Smoke";
	stateTransitionOnTriggerUp[2] = "Smoke";
	stateWaitForTimeout[2]		= false;
	stateTimeoutValue[2]            = 0.02;
	stateFire[2]                    = true;
	stateAllowImageChange[2]        = false;
	stateScript[2]                  = "onFire";
	stateSequence[2]		= "Fire";
	stateEmitter[2]			= gunFlashEmitter;
	stateEmitterTime[2]		= 0.05;
	stateEmitterNode[2]		= "muzzleNode";
	stateSound[2]			= SniperCarbinefireSound;

	stateName[3] 			= "Smoke";
	stateEmitter[3]			= gunSmokeEmitter;
	stateEmitterTime[3]		= 0.1;
	stateEmitterNode[3]		= "muzzleNode";
	stateTransitionOnTriggerUp[3] = "Wait";

	stateName[4]			= "Wait";
	stateEjectShell[2]              = true;
	stateTimeoutValue[4]		= 0.001;
	stateScript[4]                  = "onBounce";
	stateTransitionOnTimeout[4]	= "LoadCheckA";
	//stateSound[4]			= SniperCarbineClickSound;
	
	//Torque switches states instantly if there is an ammo/noammo state, regardless of stateWaitForTimeout
	stateName[5]				= "LoadCheckA";
	stateScript[5]				= "TT_onLoadCheck";
	stateTimeoutValue[5]			= 0.01;
	stateTransitionOnTimeout[5]		= "LoadCheckB";
	
	stateName[6]				= "LoadCheckB";
	stateTransitionOnAmmo[6]		= "Ready";
	stateTransitionOnNoAmmo[6]		= "Empty";
	
	stateName[7]				= "ReloadWait";
	stateTimeoutValue[7]			= 0.3;
	stateTransitionOnTimeout[7]		= "ReloadStart";
	
	stateName[8]				= "ReloadStart";
	stateTimeoutValue[8]			= 1.8;
	stateScript[8]				= "onReloadStart";
	stateTransitionOnTimeout[8]		= "Reloaded";
	
	stateName[9]				= "Reloaded";
	stateTimeoutValue[9]			= 0.7;
	stateScript[9]				= "onReloaded";
	stateTransitionOnTimeout[9]		= "Ready";

	stateName[10]                    = "FireCheckA";
	stateScript[10]                  = "TT_onFireCheck";
	stateTransitionOnTimeout[10]     = "FireCheckB";

	stateName[11]                    = "FireCheckB";
	stateTransitionOnLoaded[11]   = "Fire";
	stateTransitionOnNotLoaded[11]      = "EmptyFire";

	stateName[12]                   = "Empty";
	stateTransitionOnLoaded[12]      = "Ready";
	stateTransitionOnAmmo[12]  = "ReloadWait";
	stateTransitionOnTriggerDown[12]     = "FireCheckA";

	stateName[13]                   = "EmptyFire";
	stateScript[13]                 = "TT_onEmptyFire";
	stateTransitionOnLoaded[13]      = "Ready";
	stateTransitionOnAmmo[13]  = "ReloadWait";
	stateTransitionOnTriggerUp[13]     = "Empty";
};

// Potential change: if scoped in and SetItem to unscoped, retain image ammo
function SniperCarbineImage::onMount(%this,%obj,%slot)
{
	%temp = %obj.TT_forceToolReload;
	// onMount will reset TT_forceToolReload so we need to set it back
   Parent::onMount(%this,%obj,%slot);
   %obj.TT_forceToolReload = %temp;
}

function SniperCarbineImage::onFire(%this,%obj,%slot)
{
	if($Pref::Server::TT::Recoil)
		%obj.spawnExplosion(TTBigRecoilProjectile,"1 1 1");
	
	Parent::onFire(%this,%obj,%slot);
	%this.TT_decrementAmmo(%obj);
	%this.TT_displayAmmo(%obj);
	%obj.playThread(2, shiftAway);
}

function SniperCarbineImage::onReloadStart(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return;
	%this.TT_displayAmmo(%obj);
	%obj.playThread(2, shiftUp);
	serverPlay3D(block_MoveBrick_Sound,%obj.getPosition());
}

function SniperCarbineImage::onBounce(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return;
	%this.TT_displayAmmo(%obj);
	%obj.playThread(2, plant);
}

// function SniperCarbineImage::onReady(%this,%obj,%slot)
// {
// 	%this.TT_displayAmmo(%obj);
// }

function SniperCarbineImage::onReloaded(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return %obj.setImageLoaded(%slot, 1);
	%this.TT_reload(%obj, %slot, reloadClick8Sound, plant);
	%this.TT_displayAmmo(%obj);
}

function SniperCarbineImage::onAltTrigger(%this, %obj, %objDB, %triggerSlot, %val)
{
	if(%val && %triggerSlot == 4)
	{
		%obj.TT_retainImageAmmo = 1;
		%obj.mountImage(SniperCZoomedImage, 0);
	}
}

function SniperCarbineImage::TT_onLoadCheck(%this,%obj,%slot)
{
	if(%obj.TT_forceToolReload || %this.TT_needsAmmo(%obj))
		%obj.setImageLoaded(%slot, 0);
	else
		%obj.setImageLoaded(%slot, 1);

	if(%this.TT_canStartReload(%obj))
		%obj.setImageAmmo(%slot, 1);
	else
		%obj.setImageAmmo(%slot, 0);

	%obj.TT_forceToolReload = "";
}

function SniperCarbineItem::onUse(%this,%obj,%toolNum)
{
	%obj.TT_forceToolReload = "";
	parent::onUse(%this,%obj,%toolNum);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//sniper carbine image II: 'zoomed in' version

datablock ParticleData(SniperCarbineLaserParticle)
{
   dragCoefficient      = 5;
   gravityCoefficient   = -0.0;
   inheritedVelFactor   = 0;
   constantAcceleration = 0.0;
   lifetimeMS           = 66;
   lifetimeVarianceMS   = 0;
   textureName          = "base/data/particles/dot";
   spinSpeed           = 0.0;
   spinRandomMin        = 0.0;
   spinRandomMax        = 0.0;
   colors[0]     = "1 0.5 0.5 1";
   colors[1]     = "1 1 1 0.2";
   colors[2]     = "1 1 1 0";

   sizes[0]      = 0.35;
   sizes[1]      = 0.05;
   sizes[2]      = 0.05;

   times[0] = 0.0;
   times[1] = 0.1;
   times[2] = 1.0;

   useInvAlpha = false;
};

datablock ParticleEmitterData(SniperCarbineLaserEmitter)
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 88.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 0;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "SniperCarbineLaserParticle";
};


////////////////
//weapon image//
////////////////
datablock ShapeBaseImageData(SniperCZoomedImage)
{

   // Basic Item properties
   shapeFile = "./sniper_carbine.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
eyeoffset = "0 0.5 -0.2629";
   rotation = eulerToMatrix( "0 0 0" );

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = true;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = SniperCarbineItem;
   ammo = " ";
   projectile = SniperCarbineTracerProjectile;
   projectileType = Projectile;

   casing = GunShellDebris;
   shellExitDir        = "1.0 0.1 1.0";
   shellExitOffset     = "0 0 0";
   shellExitVariance   = 10.0;	
   shellVelocity       = 5.0;

   //melee particles shoot from eye node for consistancy
   melee = false;
   //raise your arm up or not
   armReady = true;

   doColorShift = true;
   colorShiftColor = SniperCarbineItem.colorShiftColor;

   // hook into Support_ImageAltTrigger
   altTriggerEnabled = true;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.
   TT_raycastEnabled = true;
   TT_raycastWeaponRange = 70; //varies
   TT_raycastWeaponTargets =
                   $TypeMasks::PlayerObjectType |    //AI/Players
                   $TypeMasks::StaticObjectType |    //Static Shapes
                   $TypeMasks::TerrainObjectType |    //Terrain
                   $TypeMasks::VehicleObjectType |    //Terrain
                   $TypeMasks::FXBrickObjectType;    //Bricks
   TT_raycastExplosionProjectile = GunProjectile;
   TT_raycastExplosionBrickSound = bulletHitSound;
   TT_raycastExplosionPlayerSound = bulletHitSound;
   TT_raycastDirectDamage = 14; //10
   TT_raycastDirectDamageType = $DamageType::SniperCZoomed;
   TT_raycastSpreadAmt = 0.0001; //varies
   TT_raycastSpreadCount = 1;
   TT_raycastTracerProjectile = SniperCarbineTracerProjectile;
   TT_raycastFromMuzzle = true;

   // Initial start up state
	stateName[0]                    = "Activate";
	stateTimeoutValue[0]            = 0.15;
	stateSequence[0]		= "Activate";
	stateTransitionOnTimeout[0]     = "LoadCheckA";
	stateSound[0]			= weaponSwitchSound;

	stateName[1]                    = "Ready";
	stateTransitionOnNotLoaded[1]	= "ReloadStart";
	stateTransitionOnTimeout[1]      = "Ready";
	stateTimeoutValue[1]             = 0.05;
	stateWaitForTimeout[1]           = false;
	stateTransitionOnTriggerDown[1]  = "FireCheckA";
	// stateSequence[1]	= "Ready";
	stateEmitter[1]			= SniperCarbineLaserEmitter;
	stateEmitterTime[1]		= 0.05;
	stateEmitterNode[1]		= "laserNode";
	stateScript[1]                  = "onReady";

	stateName[2]                    = "Fire";
	stateTransitionOnTimeout[2]     = "Smoke";
	stateTransitionOnTriggerUp[2] = "Smoke";
	stateWaitForTimeout[2]		= false;
	stateTimeoutValue[2]            = 0.2;
	stateFire[2]                    = true;
	stateAllowImageChange[2]        = false;
	stateScript[2]                  = "onFire";
	stateSequence[2]		= "Activate";
	stateEmitter[2]			= gunFlashEmitter;
	stateEmitterTime[2]		= 0.05;
	stateEmitterNode[2]		= "muzzleNode";
	stateSound[2]			= SniperCarbinefireSound;

	stateName[3] 			= "Smoke";
	stateEmitter[3]			= gunSmokeEmitter;
	stateEmitterTime[3]		= 0.1;
	stateEmitterNode[3]		= "muzzleNode";
	stateTransitionOnTriggerUp[3] = "Wait";

	stateName[4]			= "Wait";
	stateEjectShell[2]              = true;
	stateTimeoutValue[4]		= 0.001;
	stateScript[4]                  = "onBounce";
	stateTransitionOnTimeout[4]	= "LoadCheckA";
	//stateSound[4]			= SniperCZoomedClickSound;
	
	//Torque switches states instantly if there is an ammo/noammo state, regardless of stateWaitForTimeout
	stateName[5]				= "LoadCheckA";
	stateScript[5]				= "TT_onLoadCheck";
	stateTimeoutValue[5]			= 0.01;
	stateTransitionOnTimeout[5]		= "LoadCheckB";
	
	stateName[6]				= "LoadCheckB";
	stateTransitionOnLoaded[6]		= "Ready";
	stateTransitionOnNotLoaded[6]		= "ReloadWait";
	
	stateName[7]				= "ReloadWait";
	stateTimeoutValue[7]			= 0.3;
	stateTransitionOnTimeout[7]		= "ReloadStart";
	
	stateName[8]				= "ReloadStart";
	stateScript[8]				= "onReloadStart";

	stateName[9]                    = "FireCheckA";
	stateScript[9]                  = "TT_onFireCheck";
	stateTransitionOnTimeout[9]     = "FireCheckB";

	stateName[10]                    = "FireCheckB";
	stateTransitionOnLoaded[10]   = "Fire";
	stateTransitionOnNotLoaded[10]      = "ReloadWait";
};

function SniperCZoomedImage::onMount(%this,%obj,%slot)
{
   Parent::onMount(%this,%obj,%slot);
   %obj.pushDatablock(LMGArmor.getID());
}

function SniperCZoomedImage::onUnMount(%this,%obj,%slot)
{
   Parent::onUnMount(%this,%obj,%slot);
   %obj.popDatablock(LMGArmor.getID());
}

function SniperCZoomedImage::onFire(%this,%obj,%slot)
{
	if($Pref::Server::TT::Recoil)
		%obj.spawnExplosion(TTBigRecoilProjectile,"1 1 1");
	
	Parent::onFire(%this,%obj,%slot);
	%this.TT_decrementAmmo(%obj);
	%this.TT_displayAmmo(%obj);
	%obj.playThread(2, shiftAway);
}

// Hacky solution to force a reload after mounting the unscoped image.
function SniperCZoomedImage::onReloadStart(%this,%obj,%slot)
{
	%obj.TT_retainImageAmmo = 1;
	%obj.TT_forceToolReload = 1;
	%obj.mountImage(SniperCarbineImage, 0);
}

function SniperCZoomedImage::onBounce(%this,%obj,%slot)
{
	if($Pref::Server::TT::DeathStopAnims && %obj.getDamagePercent() >= 1.0)
		return;
	%this.TT_displayAmmo(%obj);
	%obj.playThread(2, plant);
}

function SniperCZoomedImage::onAltTrigger(%this, %obj, %objDB, %triggerSlot, %val)
{
	if(%val && %triggerSlot == 4)
	{
		%obj.TT_retainImageAmmo = 1;
		%obj.mountImage(SniperCarbineImage, 0);
	}
}

// function SniperCZoomedImage::onReady(%this,%obj,%slot)
// {
// 	%this.TT_displayAmmo(%obj);
// }

function SniperCZoomedImage::TT_isRaycastCritical(%this, %obj, %slot, %col, %pos, %normal, %hit)
{
   return TT_isRaycastHeadshot(%this, %obj, %slot, %col, %pos, %normal, %hit);
}

//////////////////////////////////////////////


// package SniperCarbine
// {
// 	function servercmdDropTool(%client,%slot)
// 	{
// 		if(%client.player.getMountedImage(0) $= SniperCZoomedImage.getID())
// 		{
// 			%client.player.unmountImage(0);
// 		}
// 		return Parent::servercmdDropTool(%client,%slot);
// 	}
// };
// activatePackage(SniperCarbine);	